#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 16
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
/* Need to handle BETA=0 case by assigning y to zero outside of loop */
void ATL_UGEMV(ATL_CINT M, ATL_CINT N, const TYPE *A, ATL_CINT lda,
               const TYPE *X, TYPE *Y)
/*
 * 48x16 unrolled mvn_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N16=(N/16)*16, M48=(M/48)*48, lda16=lda*16;
   ATL_INT j;

   #ifdef BETA0
      for (j=0; j < M; j++)
         Y[j] = ATL_rzero;
   #endif
   for (j=N16; j; j -= 16, A += lda16, X += 16)
   {
      const float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda,
                  *A5=A4+lda, *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda,
                  *A10=A9+lda, *A11=A10+lda, *A12=A11+lda, *A13=A12+lda,
                  *A14=A13+lda, *A15=A14+lda;
      const register float x0=X[0], x1=X[1], x2=X[2], x3=X[3], x4=X[4], x5=X[5],
                           x6=X[6], x7=X[7], x8=X[8], x9=X[9], x10=X[10],
                           x11=X[11], x12=X[12], x13=X[13], x14=X[14],
                           x15=X[15];
      ATL_INT i;
      prefY(Y+16+16-1);
      for (i=0; i < M48; i += 48)
      {
         const float a0_0=A0[i+0], a1_0=A0[i+1], a2_0=A0[i+2], a3_0=A0[i+3],
                     a4_0=A0[i+4], a5_0=A0[i+5], a6_0=A0[i+6], a7_0=A0[i+7],
                     a8_0=A0[i+8], a9_0=A0[i+9], a10_0=A0[i+10], a11_0=A0[i+11],
                     a12_0=A0[i+12], a13_0=A0[i+13], a14_0=A0[i+14],
                     a15_0=A0[i+15], a16_0=A0[i+16], a17_0=A0[i+17],
                     a18_0=A0[i+18], a19_0=A0[i+19], a20_0=A0[i+20],
                     a21_0=A0[i+21], a22_0=A0[i+22], a23_0=A0[i+23],
                     a24_0=A0[i+24], a25_0=A0[i+25], a26_0=A0[i+26],
                     a27_0=A0[i+27], a28_0=A0[i+28], a29_0=A0[i+29],
                     a30_0=A0[i+30], a31_0=A0[i+31], a32_0=A0[i+32],
                     a33_0=A0[i+33], a34_0=A0[i+34], a35_0=A0[i+35],
                     a36_0=A0[i+36], a37_0=A0[i+37], a38_0=A0[i+38],
                     a39_0=A0[i+39], a40_0=A0[i+40], a41_0=A0[i+41],
                     a42_0=A0[i+42], a43_0=A0[i+43], a44_0=A0[i+44],
                     a45_0=A0[i+45], a46_0=A0[i+46], a47_0=A0[i+47],
                     a0_1=A1[i+0], a1_1=A1[i+1], a2_1=A1[i+2], a3_1=A1[i+3],
                     a4_1=A1[i+4], a5_1=A1[i+5], a6_1=A1[i+6], a7_1=A1[i+7],
                     a8_1=A1[i+8], a9_1=A1[i+9], a10_1=A1[i+10], a11_1=A1[i+11],
                     a12_1=A1[i+12], a13_1=A1[i+13], a14_1=A1[i+14],
                     a15_1=A1[i+15], a16_1=A1[i+16], a17_1=A1[i+17],
                     a18_1=A1[i+18], a19_1=A1[i+19], a20_1=A1[i+20],
                     a21_1=A1[i+21], a22_1=A1[i+22], a23_1=A1[i+23],
                     a24_1=A1[i+24], a25_1=A1[i+25], a26_1=A1[i+26],
                     a27_1=A1[i+27], a28_1=A1[i+28], a29_1=A1[i+29],
                     a30_1=A1[i+30], a31_1=A1[i+31], a32_1=A1[i+32],
                     a33_1=A1[i+33], a34_1=A1[i+34], a35_1=A1[i+35],
                     a36_1=A1[i+36], a37_1=A1[i+37], a38_1=A1[i+38],
                     a39_1=A1[i+39], a40_1=A1[i+40], a41_1=A1[i+41],
                     a42_1=A1[i+42], a43_1=A1[i+43], a44_1=A1[i+44],
                     a45_1=A1[i+45], a46_1=A1[i+46], a47_1=A1[i+47],
                     a0_2=A2[i+0], a1_2=A2[i+1], a2_2=A2[i+2], a3_2=A2[i+3],
                     a4_2=A2[i+4], a5_2=A2[i+5], a6_2=A2[i+6], a7_2=A2[i+7],
                     a8_2=A2[i+8], a9_2=A2[i+9], a10_2=A2[i+10], a11_2=A2[i+11],
                     a12_2=A2[i+12], a13_2=A2[i+13], a14_2=A2[i+14],
                     a15_2=A2[i+15], a16_2=A2[i+16], a17_2=A2[i+17],
                     a18_2=A2[i+18], a19_2=A2[i+19], a20_2=A2[i+20],
                     a21_2=A2[i+21], a22_2=A2[i+22], a23_2=A2[i+23],
                     a24_2=A2[i+24], a25_2=A2[i+25], a26_2=A2[i+26],
                     a27_2=A2[i+27], a28_2=A2[i+28], a29_2=A2[i+29],
                     a30_2=A2[i+30], a31_2=A2[i+31], a32_2=A2[i+32],
                     a33_2=A2[i+33], a34_2=A2[i+34], a35_2=A2[i+35],
                     a36_2=A2[i+36], a37_2=A2[i+37], a38_2=A2[i+38],
                     a39_2=A2[i+39], a40_2=A2[i+40], a41_2=A2[i+41],
                     a42_2=A2[i+42], a43_2=A2[i+43], a44_2=A2[i+44],
                     a45_2=A2[i+45], a46_2=A2[i+46], a47_2=A2[i+47],
                     a0_3=A3[i+0], a1_3=A3[i+1], a2_3=A3[i+2], a3_3=A3[i+3],
                     a4_3=A3[i+4], a5_3=A3[i+5], a6_3=A3[i+6], a7_3=A3[i+7],
                     a8_3=A3[i+8], a9_3=A3[i+9], a10_3=A3[i+10], a11_3=A3[i+11],
                     a12_3=A3[i+12], a13_3=A3[i+13], a14_3=A3[i+14],
                     a15_3=A3[i+15], a16_3=A3[i+16], a17_3=A3[i+17],
                     a18_3=A3[i+18], a19_3=A3[i+19], a20_3=A3[i+20],
                     a21_3=A3[i+21], a22_3=A3[i+22], a23_3=A3[i+23],
                     a24_3=A3[i+24], a25_3=A3[i+25], a26_3=A3[i+26],
                     a27_3=A3[i+27], a28_3=A3[i+28], a29_3=A3[i+29],
                     a30_3=A3[i+30], a31_3=A3[i+31], a32_3=A3[i+32],
                     a33_3=A3[i+33], a34_3=A3[i+34], a35_3=A3[i+35],
                     a36_3=A3[i+36], a37_3=A3[i+37], a38_3=A3[i+38],
                     a39_3=A3[i+39], a40_3=A3[i+40], a41_3=A3[i+41],
                     a42_3=A3[i+42], a43_3=A3[i+43], a44_3=A3[i+44],
                     a45_3=A3[i+45], a46_3=A3[i+46], a47_3=A3[i+47],
                     a0_4=A4[i+0], a1_4=A4[i+1], a2_4=A4[i+2], a3_4=A4[i+3],
                     a4_4=A4[i+4], a5_4=A4[i+5], a6_4=A4[i+6], a7_4=A4[i+7],
                     a8_4=A4[i+8], a9_4=A4[i+9], a10_4=A4[i+10], a11_4=A4[i+11],
                     a12_4=A4[i+12], a13_4=A4[i+13], a14_4=A4[i+14],
                     a15_4=A4[i+15], a16_4=A4[i+16], a17_4=A4[i+17],
                     a18_4=A4[i+18], a19_4=A4[i+19], a20_4=A4[i+20],
                     a21_4=A4[i+21], a22_4=A4[i+22], a23_4=A4[i+23],
                     a24_4=A4[i+24], a25_4=A4[i+25], a26_4=A4[i+26],
                     a27_4=A4[i+27], a28_4=A4[i+28], a29_4=A4[i+29],
                     a30_4=A4[i+30], a31_4=A4[i+31], a32_4=A4[i+32],
                     a33_4=A4[i+33], a34_4=A4[i+34], a35_4=A4[i+35],
                     a36_4=A4[i+36], a37_4=A4[i+37], a38_4=A4[i+38],
                     a39_4=A4[i+39], a40_4=A4[i+40], a41_4=A4[i+41],
                     a42_4=A4[i+42], a43_4=A4[i+43], a44_4=A4[i+44],
                     a45_4=A4[i+45], a46_4=A4[i+46], a47_4=A4[i+47],
                     a0_5=A5[i+0], a1_5=A5[i+1], a2_5=A5[i+2], a3_5=A5[i+3],
                     a4_5=A5[i+4], a5_5=A5[i+5], a6_5=A5[i+6], a7_5=A5[i+7],
                     a8_5=A5[i+8], a9_5=A5[i+9], a10_5=A5[i+10], a11_5=A5[i+11],
                     a12_5=A5[i+12], a13_5=A5[i+13], a14_5=A5[i+14],
                     a15_5=A5[i+15], a16_5=A5[i+16], a17_5=A5[i+17],
                     a18_5=A5[i+18], a19_5=A5[i+19], a20_5=A5[i+20],
                     a21_5=A5[i+21], a22_5=A5[i+22], a23_5=A5[i+23],
                     a24_5=A5[i+24], a25_5=A5[i+25], a26_5=A5[i+26],
                     a27_5=A5[i+27], a28_5=A5[i+28], a29_5=A5[i+29],
                     a30_5=A5[i+30], a31_5=A5[i+31], a32_5=A5[i+32],
                     a33_5=A5[i+33], a34_5=A5[i+34], a35_5=A5[i+35],
                     a36_5=A5[i+36], a37_5=A5[i+37], a38_5=A5[i+38],
                     a39_5=A5[i+39], a40_5=A5[i+40], a41_5=A5[i+41],
                     a42_5=A5[i+42], a43_5=A5[i+43], a44_5=A5[i+44],
                     a45_5=A5[i+45], a46_5=A5[i+46], a47_5=A5[i+47],
                     a0_6=A6[i+0], a1_6=A6[i+1], a2_6=A6[i+2], a3_6=A6[i+3],
                     a4_6=A6[i+4], a5_6=A6[i+5], a6_6=A6[i+6], a7_6=A6[i+7],
                     a8_6=A6[i+8], a9_6=A6[i+9], a10_6=A6[i+10], a11_6=A6[i+11],
                     a12_6=A6[i+12], a13_6=A6[i+13], a14_6=A6[i+14],
                     a15_6=A6[i+15], a16_6=A6[i+16], a17_6=A6[i+17],
                     a18_6=A6[i+18], a19_6=A6[i+19], a20_6=A6[i+20],
                     a21_6=A6[i+21], a22_6=A6[i+22], a23_6=A6[i+23],
                     a24_6=A6[i+24], a25_6=A6[i+25], a26_6=A6[i+26],
                     a27_6=A6[i+27], a28_6=A6[i+28], a29_6=A6[i+29],
                     a30_6=A6[i+30], a31_6=A6[i+31], a32_6=A6[i+32],
                     a33_6=A6[i+33], a34_6=A6[i+34], a35_6=A6[i+35],
                     a36_6=A6[i+36], a37_6=A6[i+37], a38_6=A6[i+38],
                     a39_6=A6[i+39], a40_6=A6[i+40], a41_6=A6[i+41],
                     a42_6=A6[i+42], a43_6=A6[i+43], a44_6=A6[i+44],
                     a45_6=A6[i+45], a46_6=A6[i+46], a47_6=A6[i+47],
                     a0_7=A7[i+0], a1_7=A7[i+1], a2_7=A7[i+2], a3_7=A7[i+3],
                     a4_7=A7[i+4], a5_7=A7[i+5], a6_7=A7[i+6], a7_7=A7[i+7],
                     a8_7=A7[i+8], a9_7=A7[i+9], a10_7=A7[i+10], a11_7=A7[i+11],
                     a12_7=A7[i+12], a13_7=A7[i+13], a14_7=A7[i+14],
                     a15_7=A7[i+15], a16_7=A7[i+16], a17_7=A7[i+17],
                     a18_7=A7[i+18], a19_7=A7[i+19], a20_7=A7[i+20],
                     a21_7=A7[i+21], a22_7=A7[i+22], a23_7=A7[i+23],
                     a24_7=A7[i+24], a25_7=A7[i+25], a26_7=A7[i+26],
                     a27_7=A7[i+27], a28_7=A7[i+28], a29_7=A7[i+29],
                     a30_7=A7[i+30], a31_7=A7[i+31], a32_7=A7[i+32],
                     a33_7=A7[i+33], a34_7=A7[i+34], a35_7=A7[i+35],
                     a36_7=A7[i+36], a37_7=A7[i+37], a38_7=A7[i+38],
                     a39_7=A7[i+39], a40_7=A7[i+40], a41_7=A7[i+41],
                     a42_7=A7[i+42], a43_7=A7[i+43], a44_7=A7[i+44],
                     a45_7=A7[i+45], a46_7=A7[i+46], a47_7=A7[i+47],
                     a0_8=A8[i+0], a1_8=A8[i+1], a2_8=A8[i+2], a3_8=A8[i+3],
                     a4_8=A8[i+4], a5_8=A8[i+5], a6_8=A8[i+6], a7_8=A8[i+7],
                     a8_8=A8[i+8], a9_8=A8[i+9], a10_8=A8[i+10], a11_8=A8[i+11],
                     a12_8=A8[i+12], a13_8=A8[i+13], a14_8=A8[i+14],
                     a15_8=A8[i+15], a16_8=A8[i+16], a17_8=A8[i+17],
                     a18_8=A8[i+18], a19_8=A8[i+19], a20_8=A8[i+20],
                     a21_8=A8[i+21], a22_8=A8[i+22], a23_8=A8[i+23],
                     a24_8=A8[i+24], a25_8=A8[i+25], a26_8=A8[i+26],
                     a27_8=A8[i+27], a28_8=A8[i+28], a29_8=A8[i+29],
                     a30_8=A8[i+30], a31_8=A8[i+31], a32_8=A8[i+32],
                     a33_8=A8[i+33], a34_8=A8[i+34], a35_8=A8[i+35],
                     a36_8=A8[i+36], a37_8=A8[i+37], a38_8=A8[i+38],
                     a39_8=A8[i+39], a40_8=A8[i+40], a41_8=A8[i+41],
                     a42_8=A8[i+42], a43_8=A8[i+43], a44_8=A8[i+44],
                     a45_8=A8[i+45], a46_8=A8[i+46], a47_8=A8[i+47],
                     a0_9=A9[i+0], a1_9=A9[i+1], a2_9=A9[i+2], a3_9=A9[i+3],
                     a4_9=A9[i+4], a5_9=A9[i+5], a6_9=A9[i+6], a7_9=A9[i+7],
                     a8_9=A9[i+8], a9_9=A9[i+9], a10_9=A9[i+10], a11_9=A9[i+11],
                     a12_9=A9[i+12], a13_9=A9[i+13], a14_9=A9[i+14],
                     a15_9=A9[i+15], a16_9=A9[i+16], a17_9=A9[i+17],
                     a18_9=A9[i+18], a19_9=A9[i+19], a20_9=A9[i+20],
                     a21_9=A9[i+21], a22_9=A9[i+22], a23_9=A9[i+23],
                     a24_9=A9[i+24], a25_9=A9[i+25], a26_9=A9[i+26],
                     a27_9=A9[i+27], a28_9=A9[i+28], a29_9=A9[i+29],
                     a30_9=A9[i+30], a31_9=A9[i+31], a32_9=A9[i+32],
                     a33_9=A9[i+33], a34_9=A9[i+34], a35_9=A9[i+35],
                     a36_9=A9[i+36], a37_9=A9[i+37], a38_9=A9[i+38],
                     a39_9=A9[i+39], a40_9=A9[i+40], a41_9=A9[i+41],
                     a42_9=A9[i+42], a43_9=A9[i+43], a44_9=A9[i+44],
                     a45_9=A9[i+45], a46_9=A9[i+46], a47_9=A9[i+47],
                     a0_10=A10[i+0], a1_10=A10[i+1], a2_10=A10[i+2],
                     a3_10=A10[i+3], a4_10=A10[i+4], a5_10=A10[i+5],
                     a6_10=A10[i+6], a7_10=A10[i+7], a8_10=A10[i+8],
                     a9_10=A10[i+9], a10_10=A10[i+10], a11_10=A10[i+11],
                     a12_10=A10[i+12], a13_10=A10[i+13], a14_10=A10[i+14],
                     a15_10=A10[i+15], a16_10=A10[i+16], a17_10=A10[i+17],
                     a18_10=A10[i+18], a19_10=A10[i+19], a20_10=A10[i+20],
                     a21_10=A10[i+21], a22_10=A10[i+22], a23_10=A10[i+23],
                     a24_10=A10[i+24], a25_10=A10[i+25], a26_10=A10[i+26],
                     a27_10=A10[i+27], a28_10=A10[i+28], a29_10=A10[i+29],
                     a30_10=A10[i+30], a31_10=A10[i+31], a32_10=A10[i+32],
                     a33_10=A10[i+33], a34_10=A10[i+34], a35_10=A10[i+35],
                     a36_10=A10[i+36], a37_10=A10[i+37], a38_10=A10[i+38],
                     a39_10=A10[i+39], a40_10=A10[i+40], a41_10=A10[i+41],
                     a42_10=A10[i+42], a43_10=A10[i+43], a44_10=A10[i+44],
                     a45_10=A10[i+45], a46_10=A10[i+46], a47_10=A10[i+47],
                     a0_11=A11[i+0], a1_11=A11[i+1], a2_11=A11[i+2],
                     a3_11=A11[i+3], a4_11=A11[i+4], a5_11=A11[i+5],
                     a6_11=A11[i+6], a7_11=A11[i+7], a8_11=A11[i+8],
                     a9_11=A11[i+9], a10_11=A11[i+10], a11_11=A11[i+11],
                     a12_11=A11[i+12], a13_11=A11[i+13], a14_11=A11[i+14],
                     a15_11=A11[i+15], a16_11=A11[i+16], a17_11=A11[i+17],
                     a18_11=A11[i+18], a19_11=A11[i+19], a20_11=A11[i+20],
                     a21_11=A11[i+21], a22_11=A11[i+22], a23_11=A11[i+23],
                     a24_11=A11[i+24], a25_11=A11[i+25], a26_11=A11[i+26],
                     a27_11=A11[i+27], a28_11=A11[i+28], a29_11=A11[i+29],
                     a30_11=A11[i+30], a31_11=A11[i+31], a32_11=A11[i+32],
                     a33_11=A11[i+33], a34_11=A11[i+34], a35_11=A11[i+35],
                     a36_11=A11[i+36], a37_11=A11[i+37], a38_11=A11[i+38],
                     a39_11=A11[i+39], a40_11=A11[i+40], a41_11=A11[i+41],
                     a42_11=A11[i+42], a43_11=A11[i+43], a44_11=A11[i+44],
                     a45_11=A11[i+45], a46_11=A11[i+46], a47_11=A11[i+47],
                     a0_12=A12[i+0], a1_12=A12[i+1], a2_12=A12[i+2],
                     a3_12=A12[i+3], a4_12=A12[i+4], a5_12=A12[i+5],
                     a6_12=A12[i+6], a7_12=A12[i+7], a8_12=A12[i+8],
                     a9_12=A12[i+9], a10_12=A12[i+10], a11_12=A12[i+11],
                     a12_12=A12[i+12], a13_12=A12[i+13], a14_12=A12[i+14],
                     a15_12=A12[i+15], a16_12=A12[i+16], a17_12=A12[i+17],
                     a18_12=A12[i+18], a19_12=A12[i+19], a20_12=A12[i+20],
                     a21_12=A12[i+21], a22_12=A12[i+22], a23_12=A12[i+23],
                     a24_12=A12[i+24], a25_12=A12[i+25], a26_12=A12[i+26],
                     a27_12=A12[i+27], a28_12=A12[i+28], a29_12=A12[i+29],
                     a30_12=A12[i+30], a31_12=A12[i+31], a32_12=A12[i+32],
                     a33_12=A12[i+33], a34_12=A12[i+34], a35_12=A12[i+35],
                     a36_12=A12[i+36], a37_12=A12[i+37], a38_12=A12[i+38],
                     a39_12=A12[i+39], a40_12=A12[i+40], a41_12=A12[i+41],
                     a42_12=A12[i+42], a43_12=A12[i+43], a44_12=A12[i+44],
                     a45_12=A12[i+45], a46_12=A12[i+46], a47_12=A12[i+47],
                     a0_13=A13[i+0], a1_13=A13[i+1], a2_13=A13[i+2],
                     a3_13=A13[i+3], a4_13=A13[i+4], a5_13=A13[i+5],
                     a6_13=A13[i+6], a7_13=A13[i+7], a8_13=A13[i+8],
                     a9_13=A13[i+9], a10_13=A13[i+10], a11_13=A13[i+11],
                     a12_13=A13[i+12], a13_13=A13[i+13], a14_13=A13[i+14],
                     a15_13=A13[i+15], a16_13=A13[i+16], a17_13=A13[i+17],
                     a18_13=A13[i+18], a19_13=A13[i+19], a20_13=A13[i+20],
                     a21_13=A13[i+21], a22_13=A13[i+22], a23_13=A13[i+23],
                     a24_13=A13[i+24], a25_13=A13[i+25], a26_13=A13[i+26],
                     a27_13=A13[i+27], a28_13=A13[i+28], a29_13=A13[i+29],
                     a30_13=A13[i+30], a31_13=A13[i+31], a32_13=A13[i+32],
                     a33_13=A13[i+33], a34_13=A13[i+34], a35_13=A13[i+35],
                     a36_13=A13[i+36], a37_13=A13[i+37], a38_13=A13[i+38],
                     a39_13=A13[i+39], a40_13=A13[i+40], a41_13=A13[i+41],
                     a42_13=A13[i+42], a43_13=A13[i+43], a44_13=A13[i+44],
                     a45_13=A13[i+45], a46_13=A13[i+46], a47_13=A13[i+47],
                     a0_14=A14[i+0], a1_14=A14[i+1], a2_14=A14[i+2],
                     a3_14=A14[i+3], a4_14=A14[i+4], a5_14=A14[i+5],
                     a6_14=A14[i+6], a7_14=A14[i+7], a8_14=A14[i+8],
                     a9_14=A14[i+9], a10_14=A14[i+10], a11_14=A14[i+11],
                     a12_14=A14[i+12], a13_14=A14[i+13], a14_14=A14[i+14],
                     a15_14=A14[i+15], a16_14=A14[i+16], a17_14=A14[i+17],
                     a18_14=A14[i+18], a19_14=A14[i+19], a20_14=A14[i+20],
                     a21_14=A14[i+21], a22_14=A14[i+22], a23_14=A14[i+23],
                     a24_14=A14[i+24], a25_14=A14[i+25], a26_14=A14[i+26],
                     a27_14=A14[i+27], a28_14=A14[i+28], a29_14=A14[i+29],
                     a30_14=A14[i+30], a31_14=A14[i+31], a32_14=A14[i+32],
                     a33_14=A14[i+33], a34_14=A14[i+34], a35_14=A14[i+35],
                     a36_14=A14[i+36], a37_14=A14[i+37], a38_14=A14[i+38],
                     a39_14=A14[i+39], a40_14=A14[i+40], a41_14=A14[i+41],
                     a42_14=A14[i+42], a43_14=A14[i+43], a44_14=A14[i+44],
                     a45_14=A14[i+45], a46_14=A14[i+46], a47_14=A14[i+47],
                     a0_15=A15[i+0], a1_15=A15[i+1], a2_15=A15[i+2],
                     a3_15=A15[i+3], a4_15=A15[i+4], a5_15=A15[i+5],
                     a6_15=A15[i+6], a7_15=A15[i+7], a8_15=A15[i+8],
                     a9_15=A15[i+9], a10_15=A15[i+10], a11_15=A15[i+11],
                     a12_15=A15[i+12], a13_15=A15[i+13], a14_15=A15[i+14],
                     a15_15=A15[i+15], a16_15=A15[i+16], a17_15=A15[i+17],
                     a18_15=A15[i+18], a19_15=A15[i+19], a20_15=A15[i+20],
                     a21_15=A15[i+21], a22_15=A15[i+22], a23_15=A15[i+23],
                     a24_15=A15[i+24], a25_15=A15[i+25], a26_15=A15[i+26],
                     a27_15=A15[i+27], a28_15=A15[i+28], a29_15=A15[i+29],
                     a30_15=A15[i+30], a31_15=A15[i+31], a32_15=A15[i+32],
                     a33_15=A15[i+33], a34_15=A15[i+34], a35_15=A15[i+35],
                     a36_15=A15[i+36], a37_15=A15[i+37], a38_15=A15[i+38],
                     a39_15=A15[i+39], a40_15=A15[i+40], a41_15=A15[i+41],
                     a42_15=A15[i+42], a43_15=A15[i+43], a44_15=A15[i+44],
                     a45_15=A15[i+45], a46_15=A15[i+46], a47_15=A15[i+47];
         register float y0=Y[i+0], y1=Y[i+1], y2=Y[i+2], y3=Y[i+3], y4=Y[i+4],
                        y5=Y[i+5], y6=Y[i+6], y7=Y[i+7], y8=Y[i+8], y9=Y[i+9],
                        y10=Y[i+10], y11=Y[i+11], y12=Y[i+12], y13=Y[i+13],
                        y14=Y[i+14], y15=Y[i+15], y16=Y[i+16], y17=Y[i+17],
                        y18=Y[i+18], y19=Y[i+19], y20=Y[i+20], y21=Y[i+21],
                        y22=Y[i+22], y23=Y[i+23], y24=Y[i+24], y25=Y[i+25],
                        y26=Y[i+26], y27=Y[i+27], y28=Y[i+28], y29=Y[i+29],
                        y30=Y[i+30], y31=Y[i+31], y32=Y[i+32], y33=Y[i+33],
                        y34=Y[i+34], y35=Y[i+35], y36=Y[i+36], y37=Y[i+37],
                        y38=Y[i+38], y39=Y[i+39], y40=Y[i+40], y41=Y[i+41],
                        y42=Y[i+42], y43=Y[i+43], y44=Y[i+44], y45=Y[i+45],
                        y46=Y[i+46], y47=Y[i+47];

         prefX(X);
         y0 += a0_0 * x0;
         prefA(A0);
         y1 += a1_0 * x0;
         y2 += a2_0 * x0;
         y3 += a3_0 * x0;
         y4 += a4_0 * x0;
         y5 += a5_0 * x0;
         y6 += a6_0 * x0;
         y7 += a7_0 * x0;
         y8 += a8_0 * x0;
         y9 += a9_0 * x0;
         y10 += a10_0 * x0;
         y11 += a11_0 * x0;
         y12 += a12_0 * x0;
         y13 += a13_0 * x0;
         y14 += a14_0 * x0;
         y15 += a15_0 * x0;
         y16 += a16_0 * x0;
         prefA(A0);
         y17 += a17_0 * x0;
         y18 += a18_0 * x0;
         y19 += a19_0 * x0;
         y20 += a20_0 * x0;
         y21 += a21_0 * x0;
         y22 += a22_0 * x0;
         y23 += a23_0 * x0;
         y24 += a24_0 * x0;
         y25 += a25_0 * x0;
         y26 += a26_0 * x0;
         y27 += a27_0 * x0;
         y28 += a28_0 * x0;
         y29 += a29_0 * x0;
         y30 += a30_0 * x0;
         y31 += a31_0 * x0;
         y32 += a32_0 * x0;
         prefA(A0);
         y33 += a33_0 * x0;
         y34 += a34_0 * x0;
         y35 += a35_0 * x0;
         y36 += a36_0 * x0;
         y37 += a37_0 * x0;
         y38 += a38_0 * x0;
         y39 += a39_0 * x0;
         y40 += a40_0 * x0;
         y41 += a41_0 * x0;
         y42 += a42_0 * x0;
         y43 += a43_0 * x0;
         y44 += a44_0 * x0;
         y45 += a45_0 * x0;
         y46 += a46_0 * x0;
         y47 += a47_0 * x0;
         y0 += a0_1 * x1;
         prefA(A1);
         y1 += a1_1 * x1;
         y2 += a2_1 * x1;
         y3 += a3_1 * x1;
         y4 += a4_1 * x1;
         y5 += a5_1 * x1;
         y6 += a6_1 * x1;
         y7 += a7_1 * x1;
         y8 += a8_1 * x1;
         y9 += a9_1 * x1;
         y10 += a10_1 * x1;
         y11 += a11_1 * x1;
         y12 += a12_1 * x1;
         y13 += a13_1 * x1;
         y14 += a14_1 * x1;
         y15 += a15_1 * x1;
         y16 += a16_1 * x1;
         prefA(A1);
         y17 += a17_1 * x1;
         y18 += a18_1 * x1;
         y19 += a19_1 * x1;
         y20 += a20_1 * x1;
         y21 += a21_1 * x1;
         y22 += a22_1 * x1;
         y23 += a23_1 * x1;
         y24 += a24_1 * x1;
         y25 += a25_1 * x1;
         y26 += a26_1 * x1;
         y27 += a27_1 * x1;
         y28 += a28_1 * x1;
         y29 += a29_1 * x1;
         y30 += a30_1 * x1;
         y31 += a31_1 * x1;
         y32 += a32_1 * x1;
         prefA(A1);
         y33 += a33_1 * x1;
         y34 += a34_1 * x1;
         y35 += a35_1 * x1;
         y36 += a36_1 * x1;
         y37 += a37_1 * x1;
         y38 += a38_1 * x1;
         y39 += a39_1 * x1;
         y40 += a40_1 * x1;
         y41 += a41_1 * x1;
         y42 += a42_1 * x1;
         y43 += a43_1 * x1;
         y44 += a44_1 * x1;
         y45 += a45_1 * x1;
         y46 += a46_1 * x1;
         y47 += a47_1 * x1;
         y0 += a0_2 * x2;
         prefA(A2);
         y1 += a1_2 * x2;
         y2 += a2_2 * x2;
         y3 += a3_2 * x2;
         y4 += a4_2 * x2;
         y5 += a5_2 * x2;
         y6 += a6_2 * x2;
         y7 += a7_2 * x2;
         y8 += a8_2 * x2;
         y9 += a9_2 * x2;
         y10 += a10_2 * x2;
         y11 += a11_2 * x2;
         y12 += a12_2 * x2;
         y13 += a13_2 * x2;
         y14 += a14_2 * x2;
         y15 += a15_2 * x2;
         y16 += a16_2 * x2;
         prefA(A2);
         y17 += a17_2 * x2;
         y18 += a18_2 * x2;
         y19 += a19_2 * x2;
         y20 += a20_2 * x2;
         y21 += a21_2 * x2;
         y22 += a22_2 * x2;
         y23 += a23_2 * x2;
         y24 += a24_2 * x2;
         y25 += a25_2 * x2;
         y26 += a26_2 * x2;
         y27 += a27_2 * x2;
         y28 += a28_2 * x2;
         y29 += a29_2 * x2;
         y30 += a30_2 * x2;
         y31 += a31_2 * x2;
         y32 += a32_2 * x2;
         prefA(A2);
         y33 += a33_2 * x2;
         y34 += a34_2 * x2;
         y35 += a35_2 * x2;
         y36 += a36_2 * x2;
         y37 += a37_2 * x2;
         y38 += a38_2 * x2;
         y39 += a39_2 * x2;
         y40 += a40_2 * x2;
         y41 += a41_2 * x2;
         y42 += a42_2 * x2;
         y43 += a43_2 * x2;
         y44 += a44_2 * x2;
         y45 += a45_2 * x2;
         y46 += a46_2 * x2;
         y47 += a47_2 * x2;
         y0 += a0_3 * x3;
         prefA(A3);
         y1 += a1_3 * x3;
         y2 += a2_3 * x3;
         y3 += a3_3 * x3;
         y4 += a4_3 * x3;
         y5 += a5_3 * x3;
         y6 += a6_3 * x3;
         y7 += a7_3 * x3;
         y8 += a8_3 * x3;
         y9 += a9_3 * x3;
         y10 += a10_3 * x3;
         y11 += a11_3 * x3;
         y12 += a12_3 * x3;
         y13 += a13_3 * x3;
         y14 += a14_3 * x3;
         y15 += a15_3 * x3;
         y16 += a16_3 * x3;
         prefA(A3);
         y17 += a17_3 * x3;
         y18 += a18_3 * x3;
         y19 += a19_3 * x3;
         y20 += a20_3 * x3;
         y21 += a21_3 * x3;
         y22 += a22_3 * x3;
         y23 += a23_3 * x3;
         y24 += a24_3 * x3;
         y25 += a25_3 * x3;
         y26 += a26_3 * x3;
         y27 += a27_3 * x3;
         y28 += a28_3 * x3;
         y29 += a29_3 * x3;
         y30 += a30_3 * x3;
         y31 += a31_3 * x3;
         y32 += a32_3 * x3;
         prefA(A3);
         y33 += a33_3 * x3;
         y34 += a34_3 * x3;
         y35 += a35_3 * x3;
         y36 += a36_3 * x3;
         y37 += a37_3 * x3;
         y38 += a38_3 * x3;
         y39 += a39_3 * x3;
         y40 += a40_3 * x3;
         y41 += a41_3 * x3;
         y42 += a42_3 * x3;
         y43 += a43_3 * x3;
         y44 += a44_3 * x3;
         y45 += a45_3 * x3;
         y46 += a46_3 * x3;
         y47 += a47_3 * x3;
         y0 += a0_4 * x4;
         prefA(A4);
         y1 += a1_4 * x4;
         y2 += a2_4 * x4;
         y3 += a3_4 * x4;
         y4 += a4_4 * x4;
         y5 += a5_4 * x4;
         y6 += a6_4 * x4;
         y7 += a7_4 * x4;
         y8 += a8_4 * x4;
         y9 += a9_4 * x4;
         y10 += a10_4 * x4;
         y11 += a11_4 * x4;
         y12 += a12_4 * x4;
         y13 += a13_4 * x4;
         y14 += a14_4 * x4;
         y15 += a15_4 * x4;
         y16 += a16_4 * x4;
         prefA(A4);
         y17 += a17_4 * x4;
         y18 += a18_4 * x4;
         y19 += a19_4 * x4;
         y20 += a20_4 * x4;
         y21 += a21_4 * x4;
         y22 += a22_4 * x4;
         y23 += a23_4 * x4;
         y24 += a24_4 * x4;
         y25 += a25_4 * x4;
         y26 += a26_4 * x4;
         y27 += a27_4 * x4;
         y28 += a28_4 * x4;
         y29 += a29_4 * x4;
         y30 += a30_4 * x4;
         y31 += a31_4 * x4;
         y32 += a32_4 * x4;
         prefA(A4);
         y33 += a33_4 * x4;
         y34 += a34_4 * x4;
         y35 += a35_4 * x4;
         y36 += a36_4 * x4;
         y37 += a37_4 * x4;
         y38 += a38_4 * x4;
         y39 += a39_4 * x4;
         y40 += a40_4 * x4;
         y41 += a41_4 * x4;
         y42 += a42_4 * x4;
         y43 += a43_4 * x4;
         y44 += a44_4 * x4;
         y45 += a45_4 * x4;
         y46 += a46_4 * x4;
         y47 += a47_4 * x4;
         y0 += a0_5 * x5;
         prefA(A5);
         y1 += a1_5 * x5;
         y2 += a2_5 * x5;
         y3 += a3_5 * x5;
         y4 += a4_5 * x5;
         y5 += a5_5 * x5;
         y6 += a6_5 * x5;
         y7 += a7_5 * x5;
         y8 += a8_5 * x5;
         y9 += a9_5 * x5;
         y10 += a10_5 * x5;
         y11 += a11_5 * x5;
         y12 += a12_5 * x5;
         y13 += a13_5 * x5;
         y14 += a14_5 * x5;
         y15 += a15_5 * x5;
         y16 += a16_5 * x5;
         prefA(A5);
         y17 += a17_5 * x5;
         y18 += a18_5 * x5;
         y19 += a19_5 * x5;
         y20 += a20_5 * x5;
         y21 += a21_5 * x5;
         y22 += a22_5 * x5;
         y23 += a23_5 * x5;
         y24 += a24_5 * x5;
         y25 += a25_5 * x5;
         y26 += a26_5 * x5;
         y27 += a27_5 * x5;
         y28 += a28_5 * x5;
         y29 += a29_5 * x5;
         y30 += a30_5 * x5;
         y31 += a31_5 * x5;
         y32 += a32_5 * x5;
         prefA(A5);
         y33 += a33_5 * x5;
         y34 += a34_5 * x5;
         y35 += a35_5 * x5;
         y36 += a36_5 * x5;
         y37 += a37_5 * x5;
         y38 += a38_5 * x5;
         y39 += a39_5 * x5;
         y40 += a40_5 * x5;
         y41 += a41_5 * x5;
         y42 += a42_5 * x5;
         y43 += a43_5 * x5;
         y44 += a44_5 * x5;
         y45 += a45_5 * x5;
         y46 += a46_5 * x5;
         y47 += a47_5 * x5;
         y0 += a0_6 * x6;
         prefA(A6);
         y1 += a1_6 * x6;
         y2 += a2_6 * x6;
         y3 += a3_6 * x6;
         y4 += a4_6 * x6;
         y5 += a5_6 * x6;
         y6 += a6_6 * x6;
         y7 += a7_6 * x6;
         y8 += a8_6 * x6;
         y9 += a9_6 * x6;
         y10 += a10_6 * x6;
         y11 += a11_6 * x6;
         y12 += a12_6 * x6;
         y13 += a13_6 * x6;
         y14 += a14_6 * x6;
         y15 += a15_6 * x6;
         y16 += a16_6 * x6;
         prefA(A6);
         y17 += a17_6 * x6;
         y18 += a18_6 * x6;
         y19 += a19_6 * x6;
         y20 += a20_6 * x6;
         y21 += a21_6 * x6;
         y22 += a22_6 * x6;
         y23 += a23_6 * x6;
         y24 += a24_6 * x6;
         y25 += a25_6 * x6;
         y26 += a26_6 * x6;
         y27 += a27_6 * x6;
         y28 += a28_6 * x6;
         y29 += a29_6 * x6;
         y30 += a30_6 * x6;
         y31 += a31_6 * x6;
         y32 += a32_6 * x6;
         prefA(A6);
         y33 += a33_6 * x6;
         y34 += a34_6 * x6;
         y35 += a35_6 * x6;
         y36 += a36_6 * x6;
         y37 += a37_6 * x6;
         y38 += a38_6 * x6;
         y39 += a39_6 * x6;
         y40 += a40_6 * x6;
         y41 += a41_6 * x6;
         y42 += a42_6 * x6;
         y43 += a43_6 * x6;
         y44 += a44_6 * x6;
         y45 += a45_6 * x6;
         y46 += a46_6 * x6;
         y47 += a47_6 * x6;
         y0 += a0_7 * x7;
         prefA(A7);
         y1 += a1_7 * x7;
         y2 += a2_7 * x7;
         y3 += a3_7 * x7;
         y4 += a4_7 * x7;
         y5 += a5_7 * x7;
         y6 += a6_7 * x7;
         y7 += a7_7 * x7;
         y8 += a8_7 * x7;
         y9 += a9_7 * x7;
         y10 += a10_7 * x7;
         y11 += a11_7 * x7;
         y12 += a12_7 * x7;
         y13 += a13_7 * x7;
         y14 += a14_7 * x7;
         y15 += a15_7 * x7;
         y16 += a16_7 * x7;
         prefA(A7);
         y17 += a17_7 * x7;
         y18 += a18_7 * x7;
         y19 += a19_7 * x7;
         y20 += a20_7 * x7;
         y21 += a21_7 * x7;
         y22 += a22_7 * x7;
         y23 += a23_7 * x7;
         y24 += a24_7 * x7;
         y25 += a25_7 * x7;
         y26 += a26_7 * x7;
         y27 += a27_7 * x7;
         y28 += a28_7 * x7;
         y29 += a29_7 * x7;
         y30 += a30_7 * x7;
         y31 += a31_7 * x7;
         y32 += a32_7 * x7;
         prefA(A7);
         y33 += a33_7 * x7;
         y34 += a34_7 * x7;
         y35 += a35_7 * x7;
         y36 += a36_7 * x7;
         y37 += a37_7 * x7;
         y38 += a38_7 * x7;
         y39 += a39_7 * x7;
         y40 += a40_7 * x7;
         y41 += a41_7 * x7;
         y42 += a42_7 * x7;
         y43 += a43_7 * x7;
         y44 += a44_7 * x7;
         y45 += a45_7 * x7;
         y46 += a46_7 * x7;
         y47 += a47_7 * x7;
         y0 += a0_8 * x8;
         prefA(A8);
         y1 += a1_8 * x8;
         y2 += a2_8 * x8;
         y3 += a3_8 * x8;
         y4 += a4_8 * x8;
         y5 += a5_8 * x8;
         y6 += a6_8 * x8;
         y7 += a7_8 * x8;
         y8 += a8_8 * x8;
         y9 += a9_8 * x8;
         y10 += a10_8 * x8;
         y11 += a11_8 * x8;
         y12 += a12_8 * x8;
         y13 += a13_8 * x8;
         y14 += a14_8 * x8;
         y15 += a15_8 * x8;
         y16 += a16_8 * x8;
         prefA(A8);
         y17 += a17_8 * x8;
         y18 += a18_8 * x8;
         y19 += a19_8 * x8;
         y20 += a20_8 * x8;
         y21 += a21_8 * x8;
         y22 += a22_8 * x8;
         y23 += a23_8 * x8;
         y24 += a24_8 * x8;
         y25 += a25_8 * x8;
         y26 += a26_8 * x8;
         y27 += a27_8 * x8;
         y28 += a28_8 * x8;
         y29 += a29_8 * x8;
         y30 += a30_8 * x8;
         y31 += a31_8 * x8;
         y32 += a32_8 * x8;
         prefA(A8);
         y33 += a33_8 * x8;
         y34 += a34_8 * x8;
         y35 += a35_8 * x8;
         y36 += a36_8 * x8;
         y37 += a37_8 * x8;
         y38 += a38_8 * x8;
         y39 += a39_8 * x8;
         y40 += a40_8 * x8;
         y41 += a41_8 * x8;
         y42 += a42_8 * x8;
         y43 += a43_8 * x8;
         y44 += a44_8 * x8;
         y45 += a45_8 * x8;
         y46 += a46_8 * x8;
         y47 += a47_8 * x8;
         y0 += a0_9 * x9;
         prefA(A9);
         y1 += a1_9 * x9;
         y2 += a2_9 * x9;
         y3 += a3_9 * x9;
         y4 += a4_9 * x9;
         y5 += a5_9 * x9;
         y6 += a6_9 * x9;
         y7 += a7_9 * x9;
         y8 += a8_9 * x9;
         y9 += a9_9 * x9;
         y10 += a10_9 * x9;
         y11 += a11_9 * x9;
         y12 += a12_9 * x9;
         y13 += a13_9 * x9;
         y14 += a14_9 * x9;
         y15 += a15_9 * x9;
         y16 += a16_9 * x9;
         prefA(A9);
         y17 += a17_9 * x9;
         y18 += a18_9 * x9;
         y19 += a19_9 * x9;
         y20 += a20_9 * x9;
         y21 += a21_9 * x9;
         y22 += a22_9 * x9;
         y23 += a23_9 * x9;
         y24 += a24_9 * x9;
         y25 += a25_9 * x9;
         y26 += a26_9 * x9;
         y27 += a27_9 * x9;
         y28 += a28_9 * x9;
         y29 += a29_9 * x9;
         y30 += a30_9 * x9;
         y31 += a31_9 * x9;
         y32 += a32_9 * x9;
         prefA(A9);
         y33 += a33_9 * x9;
         y34 += a34_9 * x9;
         y35 += a35_9 * x9;
         y36 += a36_9 * x9;
         y37 += a37_9 * x9;
         y38 += a38_9 * x9;
         y39 += a39_9 * x9;
         y40 += a40_9 * x9;
         y41 += a41_9 * x9;
         y42 += a42_9 * x9;
         y43 += a43_9 * x9;
         y44 += a44_9 * x9;
         y45 += a45_9 * x9;
         y46 += a46_9 * x9;
         y47 += a47_9 * x9;
         y0 += a0_10 * x10;
         prefA(A10);
         y1 += a1_10 * x10;
         y2 += a2_10 * x10;
         y3 += a3_10 * x10;
         y4 += a4_10 * x10;
         y5 += a5_10 * x10;
         y6 += a6_10 * x10;
         y7 += a7_10 * x10;
         y8 += a8_10 * x10;
         y9 += a9_10 * x10;
         y10 += a10_10 * x10;
         y11 += a11_10 * x10;
         y12 += a12_10 * x10;
         y13 += a13_10 * x10;
         y14 += a14_10 * x10;
         y15 += a15_10 * x10;
         y16 += a16_10 * x10;
         prefA(A10);
         y17 += a17_10 * x10;
         y18 += a18_10 * x10;
         y19 += a19_10 * x10;
         y20 += a20_10 * x10;
         y21 += a21_10 * x10;
         y22 += a22_10 * x10;
         y23 += a23_10 * x10;
         y24 += a24_10 * x10;
         y25 += a25_10 * x10;
         y26 += a26_10 * x10;
         y27 += a27_10 * x10;
         y28 += a28_10 * x10;
         y29 += a29_10 * x10;
         y30 += a30_10 * x10;
         y31 += a31_10 * x10;
         y32 += a32_10 * x10;
         prefA(A10);
         y33 += a33_10 * x10;
         y34 += a34_10 * x10;
         y35 += a35_10 * x10;
         y36 += a36_10 * x10;
         y37 += a37_10 * x10;
         y38 += a38_10 * x10;
         y39 += a39_10 * x10;
         y40 += a40_10 * x10;
         y41 += a41_10 * x10;
         y42 += a42_10 * x10;
         y43 += a43_10 * x10;
         y44 += a44_10 * x10;
         y45 += a45_10 * x10;
         y46 += a46_10 * x10;
         y47 += a47_10 * x10;
         y0 += a0_11 * x11;
         prefA(A11);
         y1 += a1_11 * x11;
         y2 += a2_11 * x11;
         y3 += a3_11 * x11;
         y4 += a4_11 * x11;
         y5 += a5_11 * x11;
         y6 += a6_11 * x11;
         y7 += a7_11 * x11;
         y8 += a8_11 * x11;
         y9 += a9_11 * x11;
         y10 += a10_11 * x11;
         y11 += a11_11 * x11;
         y12 += a12_11 * x11;
         y13 += a13_11 * x11;
         y14 += a14_11 * x11;
         y15 += a15_11 * x11;
         y16 += a16_11 * x11;
         prefA(A11);
         y17 += a17_11 * x11;
         y18 += a18_11 * x11;
         y19 += a19_11 * x11;
         y20 += a20_11 * x11;
         y21 += a21_11 * x11;
         y22 += a22_11 * x11;
         y23 += a23_11 * x11;
         y24 += a24_11 * x11;
         y25 += a25_11 * x11;
         y26 += a26_11 * x11;
         y27 += a27_11 * x11;
         y28 += a28_11 * x11;
         y29 += a29_11 * x11;
         y30 += a30_11 * x11;
         y31 += a31_11 * x11;
         y32 += a32_11 * x11;
         prefA(A11);
         y33 += a33_11 * x11;
         y34 += a34_11 * x11;
         y35 += a35_11 * x11;
         y36 += a36_11 * x11;
         y37 += a37_11 * x11;
         y38 += a38_11 * x11;
         y39 += a39_11 * x11;
         y40 += a40_11 * x11;
         y41 += a41_11 * x11;
         y42 += a42_11 * x11;
         y43 += a43_11 * x11;
         y44 += a44_11 * x11;
         y45 += a45_11 * x11;
         y46 += a46_11 * x11;
         y47 += a47_11 * x11;
         y0 += a0_12 * x12;
         prefA(A12);
         y1 += a1_12 * x12;
         y2 += a2_12 * x12;
         y3 += a3_12 * x12;
         y4 += a4_12 * x12;
         y5 += a5_12 * x12;
         y6 += a6_12 * x12;
         y7 += a7_12 * x12;
         y8 += a8_12 * x12;
         y9 += a9_12 * x12;
         y10 += a10_12 * x12;
         y11 += a11_12 * x12;
         y12 += a12_12 * x12;
         y13 += a13_12 * x12;
         y14 += a14_12 * x12;
         y15 += a15_12 * x12;
         y16 += a16_12 * x12;
         prefA(A12);
         y17 += a17_12 * x12;
         y18 += a18_12 * x12;
         y19 += a19_12 * x12;
         y20 += a20_12 * x12;
         y21 += a21_12 * x12;
         y22 += a22_12 * x12;
         y23 += a23_12 * x12;
         y24 += a24_12 * x12;
         y25 += a25_12 * x12;
         y26 += a26_12 * x12;
         y27 += a27_12 * x12;
         y28 += a28_12 * x12;
         y29 += a29_12 * x12;
         y30 += a30_12 * x12;
         y31 += a31_12 * x12;
         y32 += a32_12 * x12;
         prefA(A12);
         y33 += a33_12 * x12;
         y34 += a34_12 * x12;
         y35 += a35_12 * x12;
         y36 += a36_12 * x12;
         y37 += a37_12 * x12;
         y38 += a38_12 * x12;
         y39 += a39_12 * x12;
         y40 += a40_12 * x12;
         y41 += a41_12 * x12;
         y42 += a42_12 * x12;
         y43 += a43_12 * x12;
         y44 += a44_12 * x12;
         y45 += a45_12 * x12;
         y46 += a46_12 * x12;
         y47 += a47_12 * x12;
         y0 += a0_13 * x13;
         prefA(A13);
         y1 += a1_13 * x13;
         y2 += a2_13 * x13;
         y3 += a3_13 * x13;
         y4 += a4_13 * x13;
         y5 += a5_13 * x13;
         y6 += a6_13 * x13;
         y7 += a7_13 * x13;
         y8 += a8_13 * x13;
         y9 += a9_13 * x13;
         y10 += a10_13 * x13;
         y11 += a11_13 * x13;
         y12 += a12_13 * x13;
         y13 += a13_13 * x13;
         y14 += a14_13 * x13;
         y15 += a15_13 * x13;
         y16 += a16_13 * x13;
         prefA(A13);
         y17 += a17_13 * x13;
         y18 += a18_13 * x13;
         y19 += a19_13 * x13;
         y20 += a20_13 * x13;
         y21 += a21_13 * x13;
         y22 += a22_13 * x13;
         y23 += a23_13 * x13;
         y24 += a24_13 * x13;
         y25 += a25_13 * x13;
         y26 += a26_13 * x13;
         y27 += a27_13 * x13;
         y28 += a28_13 * x13;
         y29 += a29_13 * x13;
         y30 += a30_13 * x13;
         y31 += a31_13 * x13;
         y32 += a32_13 * x13;
         prefA(A13);
         y33 += a33_13 * x13;
         y34 += a34_13 * x13;
         y35 += a35_13 * x13;
         y36 += a36_13 * x13;
         y37 += a37_13 * x13;
         y38 += a38_13 * x13;
         y39 += a39_13 * x13;
         y40 += a40_13 * x13;
         y41 += a41_13 * x13;
         y42 += a42_13 * x13;
         y43 += a43_13 * x13;
         y44 += a44_13 * x13;
         y45 += a45_13 * x13;
         y46 += a46_13 * x13;
         y47 += a47_13 * x13;
         y0 += a0_14 * x14;
         prefA(A14);
         y1 += a1_14 * x14;
         y2 += a2_14 * x14;
         y3 += a3_14 * x14;
         y4 += a4_14 * x14;
         y5 += a5_14 * x14;
         y6 += a6_14 * x14;
         y7 += a7_14 * x14;
         y8 += a8_14 * x14;
         y9 += a9_14 * x14;
         y10 += a10_14 * x14;
         y11 += a11_14 * x14;
         y12 += a12_14 * x14;
         y13 += a13_14 * x14;
         y14 += a14_14 * x14;
         y15 += a15_14 * x14;
         y16 += a16_14 * x14;
         prefA(A14);
         y17 += a17_14 * x14;
         y18 += a18_14 * x14;
         y19 += a19_14 * x14;
         y20 += a20_14 * x14;
         y21 += a21_14 * x14;
         y22 += a22_14 * x14;
         y23 += a23_14 * x14;
         y24 += a24_14 * x14;
         y25 += a25_14 * x14;
         y26 += a26_14 * x14;
         y27 += a27_14 * x14;
         y28 += a28_14 * x14;
         y29 += a29_14 * x14;
         y30 += a30_14 * x14;
         y31 += a31_14 * x14;
         y32 += a32_14 * x14;
         prefA(A14);
         y33 += a33_14 * x14;
         y34 += a34_14 * x14;
         y35 += a35_14 * x14;
         y36 += a36_14 * x14;
         y37 += a37_14 * x14;
         y38 += a38_14 * x14;
         y39 += a39_14 * x14;
         y40 += a40_14 * x14;
         y41 += a41_14 * x14;
         y42 += a42_14 * x14;
         y43 += a43_14 * x14;
         y44 += a44_14 * x14;
         y45 += a45_14 * x14;
         y46 += a46_14 * x14;
         y47 += a47_14 * x14;
         y0 += a0_15 * x15;
         prefA(A15);
         y1 += a1_15 * x15;
         y2 += a2_15 * x15;
         y3 += a3_15 * x15;
         y4 += a4_15 * x15;
         y5 += a5_15 * x15;
         y6 += a6_15 * x15;
         y7 += a7_15 * x15;
         y8 += a8_15 * x15;
         y9 += a9_15 * x15;
         y10 += a10_15 * x15;
         y11 += a11_15 * x15;
         y12 += a12_15 * x15;
         y13 += a13_15 * x15;
         y14 += a14_15 * x15;
         y15 += a15_15 * x15;
         y16 += a16_15 * x15;
         prefA(A15);
         y17 += a17_15 * x15;
         y18 += a18_15 * x15;
         y19 += a19_15 * x15;
         y20 += a20_15 * x15;
         y21 += a21_15 * x15;
         y22 += a22_15 * x15;
         y23 += a23_15 * x15;
         y24 += a24_15 * x15;
         y25 += a25_15 * x15;
         y26 += a26_15 * x15;
         y27 += a27_15 * x15;
         y28 += a28_15 * x15;
         y29 += a29_15 * x15;
         y30 += a30_15 * x15;
         y31 += a31_15 * x15;
         y32 += a32_15 * x15;
         prefA(A15);
         y33 += a33_15 * x15;
         y34 += a34_15 * x15;
         y35 += a35_15 * x15;
         y36 += a36_15 * x15;
         y37 += a37_15 * x15;
         y38 += a38_15 * x15;
         y39 += a39_15 * x15;
         y40 += a40_15 * x15;
         y41 += a41_15 * x15;
         y42 += a42_15 * x15;
         y43 += a43_15 * x15;
         y44 += a44_15 * x15;
         y45 += a45_15 * x15;
         y46 += a46_15 * x15;
         y47 += a47_15 * x15;
         Y[i+0] = y0;
         Y[i+1] = y1;
         Y[i+2] = y2;
         Y[i+3] = y3;
         Y[i+4] = y4;
         Y[i+5] = y5;
         Y[i+6] = y6;
         Y[i+7] = y7;
         Y[i+8] = y8;
         Y[i+9] = y9;
         Y[i+10] = y10;
         Y[i+11] = y11;
         Y[i+12] = y12;
         Y[i+13] = y13;
         Y[i+14] = y14;
         Y[i+15] = y15;
         Y[i+16] = y16;
         Y[i+17] = y17;
         Y[i+18] = y18;
         Y[i+19] = y19;
         Y[i+20] = y20;
         Y[i+21] = y21;
         Y[i+22] = y22;
         Y[i+23] = y23;
         Y[i+24] = y24;
         Y[i+25] = y25;
         Y[i+26] = y26;
         Y[i+27] = y27;
         Y[i+28] = y28;
         Y[i+29] = y29;
         Y[i+30] = y30;
         Y[i+31] = y31;
         Y[i+32] = y32;
         Y[i+33] = y33;
         Y[i+34] = y34;
         Y[i+35] = y35;
         Y[i+36] = y36;
         Y[i+37] = y37;
         Y[i+38] = y38;
         Y[i+39] = y39;
         Y[i+40] = y40;
         Y[i+41] = y41;
         Y[i+42] = y42;
         Y[i+43] = y43;
         Y[i+44] = y44;
         Y[i+45] = y45;
         Y[i+46] = y46;
         Y[i+47] = y47;
      }
      for (i=M48; i < M; i++)
      {
         const float a0_0=A0[i], a0_1=A1[i], a0_2=A2[i], a0_3=A3[i], a0_4=A4[i],
                     a0_5=A5[i], a0_6=A6[i], a0_7=A7[i], a0_8=A8[i], a0_9=A9[i],
                     a0_10=A10[i], a0_11=A11[i], a0_12=A12[i], a0_13=A13[i],
                     a0_14=A14[i], a0_15=A15[i];
         register float y0 = Y[i];

         y0 += a0_0 * x0;
         y0 += a0_1 * x1;
         y0 += a0_2 * x2;
         y0 += a0_3 * x3;
         y0 += a0_4 * x4;
         y0 += a0_5 * x5;
         y0 += a0_6 * x6;
         y0 += a0_7 * x7;
         y0 += a0_8 * x8;
         y0 += a0_9 * x9;
         y0 += a0_10 * x10;
         y0 += a0_11 * x11;
         y0 += a0_12 * x12;
         y0 += a0_13 * x13;
         y0 += a0_14 * x14;
         y0 += a0_15 * x15;
         Y[i] = y0;
      }
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N16; j; j--, A += lda, X++)
   {
      const float *A0=A;
      const register float x0=X[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      for (i=0; i < M48; i += 48)
      {
         const float a0_0=A0[i+0], a1_0=A0[i+1], a2_0=A0[i+2], a3_0=A0[i+3],
                     a4_0=A0[i+4], a5_0=A0[i+5], a6_0=A0[i+6], a7_0=A0[i+7],
                     a8_0=A0[i+8], a9_0=A0[i+9], a10_0=A0[i+10], a11_0=A0[i+11],
                     a12_0=A0[i+12], a13_0=A0[i+13], a14_0=A0[i+14],
                     a15_0=A0[i+15], a16_0=A0[i+16], a17_0=A0[i+17],
                     a18_0=A0[i+18], a19_0=A0[i+19], a20_0=A0[i+20],
                     a21_0=A0[i+21], a22_0=A0[i+22], a23_0=A0[i+23],
                     a24_0=A0[i+24], a25_0=A0[i+25], a26_0=A0[i+26],
                     a27_0=A0[i+27], a28_0=A0[i+28], a29_0=A0[i+29],
                     a30_0=A0[i+30], a31_0=A0[i+31], a32_0=A0[i+32],
                     a33_0=A0[i+33], a34_0=A0[i+34], a35_0=A0[i+35],
                     a36_0=A0[i+36], a37_0=A0[i+37], a38_0=A0[i+38],
                     a39_0=A0[i+39], a40_0=A0[i+40], a41_0=A0[i+41],
                     a42_0=A0[i+42], a43_0=A0[i+43], a44_0=A0[i+44],
                     a45_0=A0[i+45], a46_0=A0[i+46], a47_0=A0[i+47];
         register float y0=Y[i+0], y1=Y[i+1], y2=Y[i+2], y3=Y[i+3], y4=Y[i+4],
                        y5=Y[i+5], y6=Y[i+6], y7=Y[i+7], y8=Y[i+8], y9=Y[i+9],
                        y10=Y[i+10], y11=Y[i+11], y12=Y[i+12], y13=Y[i+13],
                        y14=Y[i+14], y15=Y[i+15], y16=Y[i+16], y17=Y[i+17],
                        y18=Y[i+18], y19=Y[i+19], y20=Y[i+20], y21=Y[i+21],
                        y22=Y[i+22], y23=Y[i+23], y24=Y[i+24], y25=Y[i+25],
                        y26=Y[i+26], y27=Y[i+27], y28=Y[i+28], y29=Y[i+29],
                        y30=Y[i+30], y31=Y[i+31], y32=Y[i+32], y33=Y[i+33],
                        y34=Y[i+34], y35=Y[i+35], y36=Y[i+36], y37=Y[i+37],
                        y38=Y[i+38], y39=Y[i+39], y40=Y[i+40], y41=Y[i+41],
                        y42=Y[i+42], y43=Y[i+43], y44=Y[i+44], y45=Y[i+45],
                        y46=Y[i+46], y47=Y[i+47];

         prefX(X);
         y0 += a0_0 * x0;
         prefA(A0);
         y1 += a1_0 * x0;
         y2 += a2_0 * x0;
         y3 += a3_0 * x0;
         y4 += a4_0 * x0;
         y5 += a5_0 * x0;
         y6 += a6_0 * x0;
         y7 += a7_0 * x0;
         y8 += a8_0 * x0;
         y9 += a9_0 * x0;
         y10 += a10_0 * x0;
         y11 += a11_0 * x0;
         y12 += a12_0 * x0;
         y13 += a13_0 * x0;
         y14 += a14_0 * x0;
         y15 += a15_0 * x0;
         y16 += a16_0 * x0;
         prefA(A0);
         y17 += a17_0 * x0;
         y18 += a18_0 * x0;
         y19 += a19_0 * x0;
         y20 += a20_0 * x0;
         y21 += a21_0 * x0;
         y22 += a22_0 * x0;
         y23 += a23_0 * x0;
         y24 += a24_0 * x0;
         y25 += a25_0 * x0;
         y26 += a26_0 * x0;
         y27 += a27_0 * x0;
         y28 += a28_0 * x0;
         y29 += a29_0 * x0;
         y30 += a30_0 * x0;
         y31 += a31_0 * x0;
         y32 += a32_0 * x0;
         prefA(A0);
         y33 += a33_0 * x0;
         y34 += a34_0 * x0;
         y35 += a35_0 * x0;
         y36 += a36_0 * x0;
         y37 += a37_0 * x0;
         y38 += a38_0 * x0;
         y39 += a39_0 * x0;
         y40 += a40_0 * x0;
         y41 += a41_0 * x0;
         y42 += a42_0 * x0;
         y43 += a43_0 * x0;
         y44 += a44_0 * x0;
         y45 += a45_0 * x0;
         y46 += a46_0 * x0;
         y47 += a47_0 * x0;
         Y[i+0] = y0;
         Y[i+1] = y1;
         Y[i+2] = y2;
         Y[i+3] = y3;
         Y[i+4] = y4;
         Y[i+5] = y5;
         Y[i+6] = y6;
         Y[i+7] = y7;
         Y[i+8] = y8;
         Y[i+9] = y9;
         Y[i+10] = y10;
         Y[i+11] = y11;
         Y[i+12] = y12;
         Y[i+13] = y13;
         Y[i+14] = y14;
         Y[i+15] = y15;
         Y[i+16] = y16;
         Y[i+17] = y17;
         Y[i+18] = y18;
         Y[i+19] = y19;
         Y[i+20] = y20;
         Y[i+21] = y21;
         Y[i+22] = y22;
         Y[i+23] = y23;
         Y[i+24] = y24;
         Y[i+25] = y25;
         Y[i+26] = y26;
         Y[i+27] = y27;
         Y[i+28] = y28;
         Y[i+29] = y29;
         Y[i+30] = y30;
         Y[i+31] = y31;
         Y[i+32] = y32;
         Y[i+33] = y33;
         Y[i+34] = y34;
         Y[i+35] = y35;
         Y[i+36] = y36;
         Y[i+37] = y37;
         Y[i+38] = y38;
         Y[i+39] = y39;
         Y[i+40] = y40;
         Y[i+41] = y41;
         Y[i+42] = y42;
         Y[i+43] = y43;
         Y[i+44] = y44;
         Y[i+45] = y45;
         Y[i+46] = y46;
         Y[i+47] = y47;
      }
      for (i=M48; i < M; i++)
      {
         const float a0_0=A0[i];
         register float y0 = Y[i];

         y0 += a0_0 * x0;
         Y[i] = y0;
      }
   }
}
